@@ -84,6 +84,9 @@ main(int argc, char **argv)
     textdomain (GETTEXT_PACKAGE);
     #endif
 
+    g_thread_init(NULL);
+    gdk_threads_init();
+
     if(!gtk_init_with_args(&argc, &argv, "", entries, PACKAGE, &cli_error))
     {
         if (cli_error != NULL)
@@ -126,7 +129,9 @@ main(int argc, char **argv)
     g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
     gtk_widget_show_all (window);
 
+    GDK_THREADS_ENTER();
     gtk_main();
+    GDK_THREADS_LEAVE();
 
     g_object_unref (settings);
 
