@@ -29,6 +29,8 @@
 #include "image_list.h"
 #include "picture_viewer.h"
 #include "settings.h"
+#include "main_window.h"
+#include "marshal.h"
 
 typedef enum
 {
@@ -127,14 +129,6 @@ rstto_picture_viewer_queued_repaint (RsttoPictureViewer *viewer, gboolean refres
 static gboolean
 rstto_picture_viewer_set_scroll_adjustments(RsttoPictureViewer *, GtkAdjustment *, GtkAdjustment *);
 
-static void
-rstto_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
-                                   GValue       *return_value,
-                                   guint         n_param_values,
-                                   const GValue *param_values,
-                                   gpointer      invocation_hint,
-                                   gpointer      marshal_data);
-
 static void
 cb_rstto_picture_viewer_value_changed(GtkAdjustment *, RsttoPictureViewer *);
 static void
@@ -219,54 +213,6 @@ rstto_picture_viewer_init(RsttoPictureViewer *viewer)
                       GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE | GDK_ACTION_PRIVATE);
 }
 
-/**
- * rstto_marshal_VOID__OBJECT_OBJECT:
- * @closure:
- * @return_value:
- * @n_param_values:
- * @param_values:
- * @invocation_hint:
- * @marshal_data:
- *
- * A marshaller for the set_scroll_adjustments signal.
- */
-static void
-rstto_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
-                                   GValue       *return_value,
-                                   guint         n_param_values,
-                                   const GValue *param_values,
-                                   gpointer      invocation_hint,
-                                   gpointer      marshal_data)
-{
-    typedef void (*GMarshalFunc_VOID__OBJECT_OBJECT) (gpointer data1,
-                                                      gpointer arg_1,
-                                                      gpointer arg_2,
-                                                      gpointer data2);
-    register GMarshalFunc_VOID__OBJECT_OBJECT callback;
-    register GCClosure *cc = (GCClosure*) closure;
-    register gpointer data1, data2;
-
-    g_return_if_fail (n_param_values == 3);
-
-    if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-        data1 = closure->data;
-        data2 = g_value_get_object (param_values + 0);
-    }
-    else
-    {
-        data1 = g_value_get_object (param_values + 0);
-        data2 = closure->data;
-    }
-    callback = (GMarshalFunc_VOID__OBJECT_OBJECT) (marshal_data ?
-    marshal_data : cc->callback);
-
-    callback (data1,
-              g_value_get_object (param_values + 1),
-              g_value_get_object (param_values + 2),
-              data2);
-}
-
 /**
  * rstto_picture_viewer_class_init:
  * @viewer_class:
@@ -771,6 +717,9 @@ rstto_picture_viewer_set_scale (RsttoPictureViewer *viewer, gdouble scale)
              * since the old and new values are required in the above code
              */
             *img_scale = scale;
+            g_object_set_data (G_OBJECT (viewer->priv->image), "viewer-scale", img_scale);
+
+            rstto_image_load (viewer->priv->image, TRUE, scale, FALSE, NULL);
 
             rstto_picture_viewer_queued_repaint (viewer, TRUE);
         }
@@ -896,6 +845,8 @@ cb_rstto_picture_viewer_scroll_event (RsttoPictureViewer *viewer, GdkEventScroll
 
                     break;
             }
+
+            rstto_picture_viewer_set_scale (viewer, *p_scale);
             gtk_adjustment_value_changed(viewer->hadjustment);
             gtk_adjustment_value_changed(viewer->vadjustment);
             viewer->priv->repaint.idle_id = g_idle_add((GSourceFunc)cb_rstto_picture_viewer_queued_repaint, viewer);
@@ -1479,6 +1430,7 @@ cb_rstto_picture_viewer_button_release_event (RsttoPictureViewer *viewer, GdkEve
                             }
 
                             g_object_set_data (G_OBJECT(viewer->priv->image), "viewer-scale", scale);
+                            rstto_picture_viewer_set_scale(viewer, *scale);
 
                             if(viewer->hadjustment)
                             {
@@ -1594,8 +1546,7 @@ rstto_picture_viewer_set_zoom_mode(RsttoPictureViewer *viewer, RsttoZoomMode mod
                 g_object_set_data (G_OBJECT (viewer->priv->image), "viewer-fit-to-screen", p_fit_to_screen);
             }
             scale = rstto_picture_viewer_calculate_scale (viewer);
-            if (scale != -1.0)
-                rstto_picture_viewer_set_scale (viewer, scale);
+            rstto_picture_viewer_set_scale (viewer, scale);
             break;
         case RSTTO_ZOOM_MODE_100:
             if (viewer->priv->image)
@@ -1658,7 +1609,9 @@ rstto_picture_viewer_set_image (RsttoPictureViewer *viewer, RsttoImage *image)
             g_object_set_data (G_OBJECT (viewer->priv->image), "viewer-fit-to-screen", fit_to_screen);
         }
 
-        rstto_image_load (viewer->priv->image, FALSE, g_value_get_uint (&max_size), FALSE, NULL);
+        rstto_image_load (viewer->priv->image, FALSE, *scale, FALSE, NULL);
+
+        g_idle_add_full(G_PRIORITY_LOW, (GSourceFunc) rstto_main_window_add_file_to_recent_files, rstto_image_get_file(viewer->priv->image), NULL);
     }
     else
     {
