@@ -18,7 +18,7 @@
 
 #include <config.h>
 #include <gtk/gtk.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4util/libxfce4util.h>
 
 #include "settings.h"
@@ -35,7 +35,10 @@
 static void
 rstto_preferences_dialog_init(RsttoPreferencesDialog *);
 static void
-rstto_preferences_dialog_class_init(RsttoPreferencesDialogClass *);
+rstto_preferences_dialog_class_init(GObjectClass *);
+
+static void
+rstto_preferences_dialog_dispose (GObject *object);
 
 
 static void
@@ -50,8 +53,6 @@ cb_rstto_preferences_dialog_cache_preload_check_button_toggled (GtkToggleButton
 static void
 cb_rstto_preferences_dialog_cache_spin_button_value_changed (GtkSpinButton *, gpointer);
 static void
-cb_rstto_preferences_dialog_image_quality_combo_box_changed (GtkComboBox *, gpointer);
-static void
 cb_rstto_preferences_dialog_image_preview_toggled (GtkToggleButton *button, 
                                                       gpointer user_data);
 static void
@@ -90,12 +91,6 @@ struct _RsttoPreferencesDialogPriv
         GtkWidget *bgcolor_color_button;
         GtkWidget *bgcolor_override_check_button;
 
-
-        GtkWidget *image_quality_frame;
-        GtkWidget *image_quality_vbox;
-        GtkWidget *image_quality_hbox;
-        GtkWidget *image_quality_label;
-        GtkWidget *image_quality_combo;
         GtkWidget *image_preview_check_button;
     } display_tab;
 
@@ -176,7 +171,6 @@ rstto_preferences_dialog_get_type (void)
 static void
 rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
 {
-    guint uint_image_quality;
     guint uint_cache_size;
     gboolean bool_preload_images;
     gboolean bool_enable_cache;
@@ -208,7 +202,6 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
 
     dialog->priv->settings = rstto_settings_new ();
     g_object_get (G_OBJECT (dialog->priv->settings),
-                  "image-quality", &uint_image_quality,
                   "cache-size", &uint_cache_size,
                   "show-preview", &bool_show_preview,
                   "preload-images", &bool_preload_images,
@@ -231,7 +224,7 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
 
 /** Bg-color frame */
     dialog->priv->display_tab.bgcolor_vbox = gtk_vbox_new (FALSE, 0);
-    dialog->priv->display_tab.bgcolor_frame = xfce_create_framebox_with_content (_("Background color"),
+    dialog->priv->display_tab.bgcolor_frame = xfce_gtk_frame_box_new_with_content(_("Background color"),
                                                                                  dialog->priv->display_tab.bgcolor_vbox);
     gtk_box_pack_start (GTK_BOX (display_main_vbox), dialog->priv->display_tab.bgcolor_frame, FALSE, FALSE, 0);
 
@@ -261,15 +254,6 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
     g_signal_connect (G_OBJECT (dialog->priv->display_tab.bgcolor_color_button), 
                       "color-set", G_CALLBACK (cb_rstto_preferences_dialog_bgcolor_color_set), dialog);
 
-/** Image-quality frame */
-    dialog->priv->display_tab.image_quality_vbox = gtk_vbox_new(FALSE, 0);
-    dialog->priv->display_tab.image_quality_frame = xfce_create_framebox_with_content (_("Quality"),
-                                                                                 dialog->priv->display_tab.image_quality_vbox);
-    gtk_box_pack_start (GTK_BOX (display_main_vbox), dialog->priv->display_tab.image_quality_frame, FALSE, FALSE, 0);
-
-    dialog->priv->display_tab.image_quality_label = gtk_label_new (_("Maximum render quality:"));
-    dialog->priv->display_tab.image_quality_hbox= gtk_hbox_new (FALSE, 4);
-    dialog->priv->display_tab.image_quality_combo= gtk_combo_box_new_text ();
     dialog->priv->display_tab.image_preview_check_button = gtk_check_button_new_with_label (_("Show preview when loading image"));
 
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->priv->display_tab.image_preview_check_button),
@@ -277,45 +261,6 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
     g_signal_connect (G_OBJECT (dialog->priv->display_tab.image_preview_check_button), 
                       "toggled", (GCallback)cb_rstto_preferences_dialog_image_preview_toggled, dialog);
 
-    gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->priv->display_tab.image_quality_combo), _("Best"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->priv->display_tab.image_quality_combo), _("High"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->priv->display_tab.image_quality_combo), _("Medium"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->priv->display_tab.image_quality_combo), _("Low"));
-
-    gtk_box_pack_start (GTK_BOX (dialog->priv->display_tab.image_quality_vbox), 
-                                 dialog->priv->display_tab.image_quality_hbox, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (dialog->priv->display_tab.image_quality_vbox), 
-                                 dialog->priv->display_tab.image_preview_check_button, FALSE, FALSE, 0);
-
-    gtk_box_pack_start (GTK_BOX (dialog->priv->display_tab.image_quality_hbox), 
-                                 dialog->priv->display_tab.image_quality_label, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (dialog->priv->display_tab.image_quality_hbox), 
-                                 dialog->priv->display_tab.image_quality_combo, FALSE, FALSE, 0);
-    /* set current value */
-    switch (uint_image_quality-(uint_image_quality%1000000))
-    {
-        case 0:
-            gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->display_tab.image_quality_combo), 0);
-            break;
-        case 8000000:
-            gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->display_tab.image_quality_combo), 1);
-            break;
-        case 4000000:
-            gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->display_tab.image_quality_combo), 2);
-            break;
-        case 2000000:
-            gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->display_tab.image_quality_combo), 3);
-            break;
-        default:
-            gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->display_tab.image_quality_combo), 2);
-            break;
-    }
-
-    /* connect signals */
-    g_signal_connect (G_OBJECT (dialog->priv->display_tab.image_quality_combo), 
-                      "changed", (GCallback)cb_rstto_preferences_dialog_image_quality_combo_box_changed, dialog);
-    
-
 /*******************/
 /** Slideshow tab **/
 /*******************/
@@ -324,7 +269,7 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), slideshow_main_vbox, slideshow_main_lbl);
 
     dialog->priv->slideshow_tab.timeout_vbox = gtk_vbox_new(FALSE, 0);
-    dialog->priv->slideshow_tab.timeout_frame = xfce_create_framebox_with_content (_("Timeout"), dialog->priv->slideshow_tab.timeout_vbox);
+    dialog->priv->slideshow_tab.timeout_frame = xfce_gtk_frame_box_new_with_content(_("Timeout"), dialog->priv->slideshow_tab.timeout_vbox);
     gtk_box_pack_start (GTK_BOX (slideshow_main_vbox), dialog->priv->slideshow_tab.timeout_frame, FALSE, FALSE, 0);
 
     timeout_lbl = gtk_label_new(_("The time period an individual image is displayed during a slideshow\n(in seconds)"));
@@ -342,7 +287,7 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
 
     
     dialog->priv->slideshow_tab.thumbnail_vbox = gtk_vbox_new(FALSE, 0);
-    dialog->priv->slideshow_tab.thumbnail_frame = xfce_create_framebox_with_content (_("Thumbnails"), dialog->priv->slideshow_tab.thumbnail_vbox);
+    dialog->priv->slideshow_tab.thumbnail_frame = xfce_gtk_frame_box_new_with_content(_("Thumbnails"), dialog->priv->slideshow_tab.thumbnail_vbox);
     gtk_box_pack_start (GTK_BOX (slideshow_main_vbox), dialog->priv->slideshow_tab.thumbnail_frame, FALSE, FALSE, 0);
 
     dialog->priv->slideshow_tab.hide_thumbnails_fullscreen_lbl = gtk_label_new(_("The thumbnailbar can be automatically hidden \nwhen the image-viewer is fullscreen."));
@@ -364,7 +309,7 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), control_main_vbox, control_main_lbl);
 
     dialog->priv->control_tab.scroll_vbox = gtk_vbox_new(FALSE, 0);
-    dialog->priv->control_tab.scroll_frame = xfce_create_framebox_with_content (_("Scrollwheel"), dialog->priv->control_tab.scroll_vbox);
+    dialog->priv->control_tab.scroll_frame = xfce_gtk_frame_box_new_with_content(_("Scrollwheel"), dialog->priv->control_tab.scroll_vbox);
     gtk_box_pack_start (GTK_BOX (control_main_vbox), dialog->priv->control_tab.scroll_frame, FALSE, FALSE, 0);
 
     dialog->priv->control_tab.no_scrollwheel_action_radio_button = gtk_radio_button_new_with_label (NULL, _("No action"));
@@ -413,7 +358,7 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
 
     /********************************************/
     dialog->priv->behaviour_tab.scaling_vbox = gtk_vbox_new(FALSE, 0);
-    dialog->priv->behaviour_tab.scaling_frame = xfce_create_framebox_with_content (_("Scaling"), dialog->priv->behaviour_tab.scaling_vbox);
+    dialog->priv->behaviour_tab.scaling_frame = xfce_gtk_frame_box_new_with_content(_("Scaling"), dialog->priv->behaviour_tab.scaling_vbox);
     gtk_box_pack_start (GTK_BOX (behaviour_main_vbox), dialog->priv->behaviour_tab.scaling_frame, FALSE, FALSE, 0);
     /* not used */
     gtk_widget_set_sensitive (dialog->priv->behaviour_tab.scaling_vbox, FALSE);
@@ -422,7 +367,7 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
     gtk_container_add (GTK_CONTAINER (dialog->priv->behaviour_tab.scaling_vbox), dialog->priv->behaviour_tab.resize_image_on_maximize);
 
     dialog->priv->behaviour_tab.startup_vbox = gtk_vbox_new(FALSE, 0);
-    dialog->priv->behaviour_tab.startup_frame = xfce_create_framebox_with_content (_("Startup"), dialog->priv->behaviour_tab.startup_vbox);
+    dialog->priv->behaviour_tab.startup_frame = xfce_gtk_frame_box_new_with_content(_("Startup"), dialog->priv->behaviour_tab.startup_vbox);
     gtk_box_pack_start (GTK_BOX (behaviour_main_vbox), dialog->priv->behaviour_tab.startup_frame, FALSE, FALSE, 0);
     dialog->priv->behaviour_tab.resize_window_on_startup_check_button = gtk_check_button_new_with_label (_("Resize window to image on startup"));
     gtk_container_add (GTK_CONTAINER (dialog->priv->behaviour_tab.startup_vbox), dialog->priv->behaviour_tab.resize_window_on_startup_check_button);
@@ -455,7 +400,7 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
 
 /** Image-cache frame */
     dialog->priv->cache_tab.cache_vbox = gtk_vbox_new(FALSE, 0);
-    dialog->priv->cache_tab.cache_frame = xfce_create_framebox_with_content (_("Image cache"),
+    dialog->priv->cache_tab.cache_frame = xfce_gtk_frame_box_new_with_content(_("Image cache"),
                                                                                  dialog->priv->cache_tab.cache_vbox);
     gtk_box_pack_start (GTK_BOX (cache_main_vbox), dialog->priv->cache_tab.cache_frame, FALSE, FALSE, 0);
 
@@ -496,7 +441,8 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
                                   bool_enable_cache);
     gtk_widget_set_sensitive (GTK_WIDGET (dialog->priv->cache_tab.cache_sub_vbox),
                               bool_enable_cache);
-
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->priv->cache_tab.cache_preload_check_button),
+                                  bool_preload_images);
     if (uint_cache_size < RSTTO_MIN_CACHE_SIZE)
     {
         gtk_adjustment_set_value (GTK_ADJUSTMENT (cache_adjustment), RSTTO_DEFAULT_CACHE_SIZE);
@@ -522,9 +468,24 @@ rstto_preferences_dialog_init(RsttoPreferencesDialog *dialog)
 }
 
 static void
-rstto_preferences_dialog_class_init(RsttoPreferencesDialogClass *dialog_class)
+rstto_preferences_dialog_class_init(GObjectClass *object_class)
 {
-    parent_class = g_type_class_peek_parent (dialog_class);
+    parent_class = g_type_class_peek_parent (RSTTO_PREFERENCES_DIALOG_CLASS (object_class));
+
+    object_class->dispose = rstto_preferences_dialog_dispose;
+}
+
+static void
+rstto_preferences_dialog_dispose (GObject *object)
+{
+    RsttoPreferencesDialog *dialog = RSTTO_PREFERENCES_DIALOG (object);
+    if (dialog->priv->settings)
+    {
+        g_object_unref (dialog->priv->settings);
+        dialog->priv->settings = NULL;
+    }
+    
+    G_OBJECT_CLASS(parent_class)->dispose(object);
 }
 
 GtkWidget *
@@ -640,38 +601,6 @@ cb_rstto_preferences_dialog_cache_spin_button_value_changed (GtkSpinButton *butt
     g_value_unset (&value);
 }
 
-static void
-cb_rstto_preferences_dialog_image_quality_combo_box_changed (GtkComboBox *combo_box,
-                                                             gpointer user_data)
-
-{
-    /* FIXME */
-    RsttoPreferencesDialog *dialog = RSTTO_PREFERENCES_DIALOG (user_data);
-    switch (gtk_combo_box_get_active (combo_box))
-    {
-        case 0: /* unlimited */
-            g_object_set (G_OBJECT (dialog->priv->settings),
-                          "image-quality", 0,
-                          NULL);
-            break;
-        case 1: /* 1 MegaPixel */
-            g_object_set (G_OBJECT (dialog->priv->settings),
-                          "image-quality", 8000000,
-                          NULL);
-            break;
-        case 2: /* 2 MegaPixel */
-            g_object_set (G_OBJECT (dialog->priv->settings),
-                          "image-quality", 4000000,
-                          NULL);
-            break;
-        case 3: /* 4 MegaPixel */
-            g_object_set (G_OBJECT (dialog->priv->settings),
-                          "image-quality", 2000000,
-                          NULL);
-            break;
-    }
-}
-
 static void
 cb_rstto_preferences_dialog_image_preview_toggled (GtkToggleButton *button, 
                                                       gpointer user_data)
